package org.test;

import java.util.Hashtable;

import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.ObjectMessage;
import javax.jms.Session;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicSession;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

public class MessageConsumer {

    private static final String HOST_NAME = "localhost";
    private static final String PORT = "1099";
    private static final String CONNECTION_FACTORY = "ThroughputConnectionFactory";
    private static final String TOPIC_NAME = "topic/A";

    public static final void main(String[] args){
        try {
            MessageConsumer consumer = new MessageConsumer();
            consumer.run();
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    public void run() throws Exception {
        listenToTopic(new TestMessageListener());
       
        
    }

    public void listenToTopic( MessageListener listener )
    throws Exception {
        TopicConnection connection = null;
        try {
            Context context = getInitialContext(HOST_NAME, PORT);
            TopicConnectionFactory factory = (TopicConnectionFactory) context
            .lookup(CONNECTION_FACTORY);
            connection = factory.createTopicConnection();
            TopicSession session = connection.createTopicSession(false,
                    Session.AUTO_ACKNOWLEDGE);
            Topic listenTopic = (Topic) context.lookup(TOPIC_NAME);


            session.createSubscriber(listenTopic).setMessageListener(listener);
            connection.start();
            System.in.read();
        } finally {
            if ( connection != null ){
                connection.close();
            }
        }

    }

    public Context getInitialContext(String host, String jnpPort)
    throws NamingException {
        Context context = null;
        Hashtable<Object, Object> jndiProps = new Hashtable<Object, Object>();

        String serverUrl = "jnp://" + host + ":" + jnpPort;

        jndiProps.put(Context.INITIAL_CONTEXT_FACTORY,
        "org.jnp.interfaces.NamingContextFactory");
        jndiProps.put(Context.URL_PKG_PREFIXES,
        "org.jboss.naming:org.jnp.interfaces");
        jndiProps.put(Context.PROVIDER_URL, serverUrl);

        context = new InitialContext(jndiProps);

        return context;
    }

    private class TestMessageListener implements MessageListener {

        public void onMessage(Message msg) {
            if (msg instanceof ObjectMessage) {
                try {
                    TestObject obj = (TestObject) ((ObjectMessage) msg).getObject();
                    System.out.println(obj);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }

        }

    }

}
